#pragma rtGlobals=1		// Use modern global access method.

Function AgeModelPlot([PlotName])
	string PlotName
	
	if(paramIsDefault(PlotName))
		PlotName=UniqueName("AgeModelPlot",6,0)
	endif
	DoWindow/K $PlotName
	Display/K=1/W=(705,44,1225,744)/N=$PlotName

	ControlBar 150
	ListBox AgeDataDisplay proc=ADPListBox
	PopupMenu InterpType pos={120,127},proc=ADPPopup,title="Interpolation:",value="Linear"
	Button DataImport pos={5,127},size={100,20},proc=ADPButton,title="Import data"

	ADP_Synchronize(PlotName)
	ADP_UpdatePlot(PlotName,new=1)
	GenericTabs("",0)
end

Function DataSelectionPanel(PlotName)
	string PlotName
	
	string PanelName=PlotName+"#AgeModelDataImport"
	NewPanel/K=2/N=AgeModelDataImport/HOST=$PlotName/W=(0,0,520,350)
	WaveBrowserListbox("DatumType",PanelName,Xpos=5,Ypos=5,Xsize=150,Ysize=150,promptText="Datum type wave",WaveOpts="TEXT:1,DIMS:1")
	WaveBrowserListbox("DatumDescription",PanelName,Xpos=5,Ypos=160,Xsize=150,Ysize=150,promptText="Datum description wave",WaveOpts="TEXT:1,DIMS:1")
	WaveBrowserListbox("DatumAgeMax",PanelName,Xpos=160,Ypos=5,Xsize=150,Ysize=150,promptText="Datum max age",WaveOpts="CMPLX:0,TEXT:0,INTEGER:0,DIMS:1,UNSIGNED:0,WORD:0")
	WaveBrowserListbox("DatumAgeMin",PanelName,Xpos=160,Ypos=160,Xsize=150,Ysize=150,promptText="Datum min age",WaveOpts="CMPLX:0,TEXT:0,INTEGER:0,DIMS:1,UNSIGNED:0,WORD:0")
	WaveBrowserListbox("DatumLevelMax",PanelName,Xpos=320,Ypos=5,Xsize=150,Ysize=150,promptText="Datum max level",WaveOpts="CMPLX:0,TEXT:0,INTEGER:0,DIMS:1,UNSIGNED:0,WORD:0")
	WaveBrowserListbox("DatumLevelMin",PanelName,Xpos=320,Ypos=160,Xsize=150,Ysize=150,promptText="Datum min level",WaveOpts="CMPLX:0,TEXT:0,INTEGER:0,DIMS:1,UNSIGNED:0,WORD:0")
	Button DataReplace pos={5,320},size={100,20},proc=ADPButton,title="Replace data"
	Button DataAppend pos={110,320},size={100,20},proc=ADPButton,title="Append data"
	Button DataImportCancel pos={215,320},size={100,20},proc=ADPButton,title="Cancel"
end

Function ADPPopup(ctrlName,popNum,popStr) : PopupMenuControl
	string ctrlName,popStr
	variable popNum
	
	
end

Function ADP_UpdatePlot(PlotName[,new])
	string PlotName
	variable new
	
	string AgeModelFolder
	
	AgeModelFolder="root:Packages:AgeModel:"+PlotName
	wave/t Type=$(AgeModelFolder+":Type"),Description=$(AgeModelFolder+":Description")
	wave Ages=$(AgeModelFolder+":Ages"),Levels=$(AgeModelFolder+":Levels"),AgeErr=$(AgeModelFolder+":AgeErr"),LevelErr=$(AgeModelFolder+":LevelErr")
	if(new)
		AppendToGraph/W=$PlotName Ages vs Levels
	endif
	ModifyGraph mode(Ages)=3,rgb(Ages)=(30583,30583,30583)
	ModifyGraph msize=3,zmrkNum(Ages)={$(AgeModelFolder+":MarkerNumber")}
	ModifyGraph zColor(Ages)={$(AgeModelFolder+":Mask"),1,-1,Grays}
	ErrorBars Ages BOX,wave=($(AgeModelFolder+":LevelErr"),$(AgeModelFolder+":LevelErr")),wave=($(AgeModelFolder+":AgeErr"),$(AgeModelFolder+":AgeErr"))
	label bottom "Level"
	label left "Age"
	modifygraph minor=1
end

Function ADPButton(ctrlName) : ButtonControl
	string ctrlName
	
	string PlotName,PanelName,AgeModelFolder
	variable nRows,startRow,newRows
	
	PlotName=WinName(0,1)
	PanelName=(PlotName+"#AgeModelDataImport")
	ctrlName=stringfromlist(0,ctrlName,"_")
	strswitch(ctrlName)
		case "DataImport":
			DataSelectionPanel(PlotName)
			break
		case "DataReplace":
		case "DataAppend":
			AgeModelFolder="root:Packages:AgeModel:"+PlotName
			wave/t AgeDataList=$(AgeModelFolder+":AgeDataList")
			wave AgeDataSel=$(AgeModelFolder+":AgeDataSel")
			nRows=dimsize(AgeDataList,0)
			wave/t DatumType=$WBL_GetSelection("DatumType",PanelName),DatumDescription=$WBL_GetSelection("DatumDescription",PanelName)
			wave DatumAgeMax=$WBL_GetSelection("DatumAgeMax",PanelName),DatumAgeMin=$WBL_GetSelection("DatumAgeMin",PanelName)
			wave DatumLevelMax=$WBL_GetSelection("DatumLevelMax",PanelName),DatumLevelMin=$WBL_GetSelection("DatumLevelMin",PanelName)
			newRows=waveexists(DatumType) ? numpnts(DatumType) : 0
			newRows=waveexists(DatumDescription) ? max(newRows,numpnts(DatumDescription)) : newRows
			newRows=waveexists(DatumAgeMax) ? max(newRows,numpnts(DatumAgeMax)) : newRows
			newRows=waveexists(DatumAgeMin) ? max(newRows,numpnts(DatumAgeMin)) : newRows
			newRows=waveexists(DatumLevelMax) ? max(newRows,numpnts(DatumLevelMax)) : newRows
			newRows=waveexists(DatumLevelMin) ? max(newRows,numpnts(DatumLevelMin)) : newRows
			if(stringMatch(ctrlName,"DataReplace"))
				startRow=0
			else
				startRow=nRows;newRows+=nRows
			endif
			redimension/n=(max(1,newRows),-1) AgeDataList,AgeDataSel
			if(waveexists(DatumType))
				AgeDataList[startRow,newRows-1][%Type]=DatumType[p-startRow]
			else
				AgeDataList[startRow,newRows-1][%Type]=""
			endif
			if(waveexists(DatumDescription))
				AgeDataList[startRow,newRows-1][%Description]=DatumDescription[p-startRow]
			else
				AgeDataList[startRow,newRows-1][%Description]=""
			endif
			AgeDataList[startRow,newRows-1][%Age1]=num2str(waveexists(DatumAgeMax) ? DatumAgeMax[p-startRow] : (waveexists(DatumAgeMin) ? DatumAgeMin[p-startRow] : nan))
			AgeDataList[startRow,newRows-1][%Age2]=num2str(waveexists(DatumAgeMin) ? DatumAgeMin[p-startRow] : (waveexists(DatumAgeMax) ? DatumAgeMax[p-startRow] : nan))
			AgeDataList[startRow,newRows-1][%Level1]=num2str(waveexists(DatumLevelMax) ? DatumLevelMax[p-startRow] : (waveexists(DatumLevelMin) ? DatumLevelMin[p-startRow] : nan))
			AgeDataList[startRow,newRows-1][%Level2]=num2str(waveexists(DatumLevelMin) ? DatumLevelMin[p-startRow] : (waveexists(DatumLevelMax) ? DatumLevelMax[p-startRow] : nan))
			AgeDataSel[][0,4]=0x02
			AgeDataSel[][5]=0x20 + 0x10
			ADP_Synchronize(PlotName)
		case "DataImportCancel":
			killwindow $PanelName
			break
	endswitch
end

Function ADP_Synchronize(PlotName)
	string PlotName
	
	string AgeModelFolder
	variable nrows,ncols=7,i
	string SymbolsStr="19;16;17;23;46;49;26;29;18",TypesStr=""
	
	newdatafolder/o root:Packages
	newdatafolder/o root:Packages:AgeModel
	AgeModelFolder="root:Packages:AgeModel:"+PlotName
	newdatafolder/o $AgeModelFolder
	if(!waveexists($(AgeModelFolder+":AgeDataList")))
		make/o/n=(1,ncols)/t $(AgeModelFolder+":AgeDataList")
		make/o/n=(1,ncols) $(AgeModelFolder+":AgeDataSel")
	endif
	wave/t AgeDataList=$(AgeModelFolder+":AgeDataList")
	wave AgeDataSel=$(AgeModelFolder+":AgeDataSel")
	nrows=max(DimSize(AgeDataList,0),1)
	redimension/n=(nrows,ncols) AgeDataList,AgeDataSel

	SetDimLabel 1,0,Type,AgeDataList,AgeDataSel
	SetDimLabel 1,1,Description,AgeDataList,AgeDataSel
	SetDimLabel 1,2,Level1,AgeDataList,AgeDataSel
	SetDimLabel 1,3,Level2,AgeDataList,AgeDataSel
	SetDimLabel 1,4,Age1,AgeDataList,AgeDataSel
	SetDimLabel 1,5,Age2,AgeDataList,AgeDataSel
	SetDimLabel 1,6,Use,AgeDataList,AgeDataSel

	AgeDataSel[][]=(q<ncols-1) ? 0x02 : ((AgeDataSel[p][q] & 0x20) ? AgeDataSel[p][q] : 0x20)

	make/o/n=(nrows)/t $(AgeModelFolder+":Type"),$(AgeModelFolder+":Description")
	make/o/n=(nrows) $(AgeModelFolder+":Ages"),$(AgeModelFolder+":Levels"),$(AgeModelFolder+":AgeErr"),$(AgeModelFolder+":LevelErr")
	make/o/n=(nrows) $(AgeModelFolder+":Mask"),$(AgeModelFolder+":MarkerNumber")
	wave/t Type=$(AgeModelFolder+":Type"),Description=$(AgeModelFolder+":Description")
	wave Ages=$(AgeModelFolder+":Ages"),Levels=$(AgeModelFolder+":Levels"),AgeErr=$(AgeModelFolder+":AgeErr"),LevelErr=$(AgeModelFolder+":LevelErr")
	wave Mask=$(AgeModelFolder+":Mask"),MarkerNumber=$(AgeModelFolder+":MarkerNumber")
	Type[]=AgeDataList[p][%Type]
	Description[]=AgeDataList[p][%Description]
	Ages[]=(str2num(AgeDataList[p][%Age1])+str2num(AgeDataList[p][%Age2]))/2
	Levels[]=(str2num(AgeDataList[p][%Level1])+str2num(AgeDataList[p][%Level2]))/2
	AgeErr[]=abs(str2num(AgeDataList[p][%Age1])-str2num(AgeDataList[p][%Age2]))/2
	LevelErr[]=abs(str2num(AgeDataList[p][%Level1])-str2num(AgeDataList[p][%Level2]))/2
	Mask[]=AgeDataSel[p][%Use] & 0x10
	
	for(i=0;i<nrows;i+=1)
		TypesStr=AddListItem(AgeDataList[i][%Type],RemoveFromList(AgeDataList[i][%Type],TypesStr),";",inf)
	endfor
	MarkerNumber[]=str2num(stringfromlist(WhichListItem(AgeDataList[p][%Type],TypesStr),SymbolsStr))
	
	ListBox AgeDataDisplay pos={10,21},size={400,106},widths={50,100,50,50,50,50,25},mode=7,editStyle=1
	ListBox AgeDataDisplay listwave=AgeDataList,selWave=$(AgeModelFolder+":AgeDataSel")
end

Function ADPListBox(AMs) : ListboxControl
	STRUCT WMListboxAction &AMs
	
	string AgeModelFolder,numWaveNames,textWaveNames
	AgeModelFolder="root:Packages:AgeModel:"+AMs.win
	
	wave/t AgeDataList=$(AgeModelFolder+":AgeDataList")
	wave AgeDataSel=$(AgeModelFolder+":AgeDataSel")
	wave/t Type=$(AgeModelFolder+":Type")
	wave Ages=$(AgeModelFolder+":Ages"),Levels=$(AgeModelFolder+":Levels"),AgeErr=$(AgeModelFolder+":AgeErr"),LevelErr=$(AgeModelFolder+":LevelErr")
	variable nrows=DimSize(AgeDataList,0),ncols=DimSize(AgeDataList,1),cellval,i
	string cellstr
	switch(AMs.eventCode)
		case 1:
		case 2:
		case 3:
			if(AMs.row==-1)
				MatrixSort(0,AMS.col,textMat=AgeDataList,alphanumeric=1,casesensitive=1,reversed=0,numMatnames=getwavesdatafolder(AgeDataSel,2))
			elseif(AMs.row>=nrows)
				nrows+=1
				redimension/n=(max(1,nrows),-1) AgeDataList,AgeDataSel,Ages,Levels,AgeErr,LevelErr
				AgeDataSel=0x02
				AgeDataSel[AMs.row][AMs.col]+=0x01
			endif
			break
		case 7:
			if(AMs.col>1)
				cellval=str2num(AgeDataList[AMs.row][AMs.col])
				cellstr=selectstring(numtype(cellval),num2str(cellval),"")
				AgeDataList[AMs.row][AMs.col]=cellstr
			endif
			if(nrows>1)
				if(strlen(AgeDataList[AMs.row][0])+strlen(AgeDataList[AMs.row][1])+strlen(AgeDataList[AMs.row][2])+strlen(AgeDataList[AMs.row][3])+strlen(AgeDataList[AMs.row][4])+strlen(AgeDataList[AMs.row][5])==0)
					deletepoints/M=0 AMs.row,1,AgeDataList,AgeDataSel
				endif
			endif
			break
	endswitch
	ADP_Synchronize(AMs.win)
end

Function linAge(depth,agew,depthw)
	variable depth;wave agew,depthw
	
	variable p1=nan,p2=nan,i,npts=numpnts(depthw)
	for(i=0;i<npts;i+=1)
		if(numtype(depthw[i])+numtype(agew[i])==0)
			if(depthw[i]<depth)
				p1=i
			else
				p2=i
				break
			endif
		endif
	endfor
	if(numtype(p1))
		p1=p2;p2=nan
		for(i=p1+1;i<npts;i+=1)
			if(numtype(depthw[i])+numtype(agew[i])==0)
				p2=i
				break
			endif
		endfor
	endif
	if(numtype(p2))
		p2=p1;p1=nan
		for(i=p2-1;i>=0;i-=1)
			if(numtype(depthw[i])+numtype(agew[i])==0)
				p1=i
				break
			endif
		endfor
	endif
	return agew[p1]+(depth-depthw[p1])*(agew[p2]-agew[p1])/(depthw[p2]-depthw[p1])
end

Function BRagefunc(d)
	variable d
	
	if(d<932)
		return nan
	elseif(d<954.4)	//Sequence E6 - ages must be >45.8, <49.7
		return 46+(d-932)*(47-46)/(953-933)
	elseif(d<959.85)	//Sequence E5 - ages constrained by <49.7 at 955.4, <50.4
		return 49.7+(d-955.4)*(50-49.7)/(959.85-955.4)
	elseif(d<1016.6)	//Sequence E4 - ages constrained by >50.6 at 985, <50.4 at 996, prob <50.8
		return 50+(d-959.85)*(50.65-50.1)/(1016.6-959.85)
	elseif(d<1108.6)	//Sequence E3 - ages constrained by >50.8 at 1049.9, <52.3 at 1105
		return 50.8+(d-1049.9)*(50.8-50.7)/(1049.9-1016.6)
	elseif(d<1134.4)	//Sequence E2 - ages constrained by <52.85 at 1119.9, <53.347
		return 52.85+(d-1119.9)*(53.3-52.85)/(1134.4-1119.9)
	elseif(d<1138.7)	//Sequence E1 - ages constrained by >53.6 at 1135.5,<54.4 at 1138.7
		return 53.6+(d-1135.5)*(54.4-53.6)/(1138.7-1135.5)
	elseif(d<1171.1)	//Sequence uPa3 - ages constrained by ~>55.5 at 1171.1, >55
		return 55.5+(d-1171.1)*(55.5-55)/(1171.1-1138.7)
	elseif(d<1240.8)	//Sequence lPa3 - ages constrained by ~>55.5 at 1171.1, ~55.904 at 1192.3
		return 55.5+(d-1171.1)*(55.904-55.5)/(1192.3-1171.1)
	elseif(d<1256.3)	//Sequence Pa2 - ages constrained by <58.4 at 1250, <59.7, >57.3
		return 58.4+(d-1250)*(58.4-57.3)/(1250-1240.8)
	elseif(d<1258.7)	//Sequence Pa1 - ages must be <63, >62.2
		return 62.2+(d-1256.3)*(62.9-62.2)/(1258.7-1256.3)
	elseif(d>1258.7)
		return nan
	endif
end

Function ANagefunc(d)
	variable d
	
	if(d<427.6)
		return nan
	elseif(d<443)	//Sequence E6 - ages constrained by <47.3 at 433.5, <46.1 at 430.5, <49.7
		return 46.1+(d-430.5)*(48-46.1)/(443-430.5)
	elseif(d<446.7)	//Sequence E5 - ages constrained by <49.7 at 446, >47.3, <<52.3
		return 49.7+(d-446)*(49.7-48)/(446-443)
	elseif(d<448.7)	//Sequence E4 - ages must be <<52.3, >49.7
		return 50+(d-446.7)*(50.4-50)/(448.7-446.7)
	elseif(d<461.2)	//Sequence E3 - ages constrained by <52.3 at 460.4, >>49.7, <52.85
		return 52.3+(d-460.4)*(52.3-50.7)/(460.4-448.7)
	elseif(d<519.2)	//Sequence E2 - ages constrained by >53.6 at 519, <52.85 at 471, >52.3
		return 52.85+(d-471)*(53.6-52.85)/(519-471)
	elseif(d<522.2)	//Sequence E1 - ages constrained by >54.2 at 519.9,<54.4 at 521,<55,>54
		return 54.4+(d-521)*(54.4-54)/(521-519.2)
	elseif(d<560)	//Sequence uPa3 - ages constrained by ~>55.5 at 560, >55
		return 55.5+(d-560)*(55.5-55)/(560-522.2)
	elseif(d<599)	//Sequence lPa3 - ages constrained by ~>55.5 at 560, <56.2 at 594
		return 55.5+(d-560)*(56.2-55.5)/(594-560)
	elseif(d<606.5)	//Sequence Pa2 - ages constrained by <61 at 606, <59.2 at 600.4, >57.3
		return 59.2+(d-600.4)*(61-59.2)/(606-600.4)
	elseif(d<612.5)	//Sequence Pa1 - ages constrained by <62.2 at 607.5, <63 at 612, >61
		return 62.2+(d-607.5)*(62.9-62.2)/(612.5-607.5)
	elseif(d>612.5)
		return nan
	endif
end

static Function MatrixSort(dim,key[,textMat,numMat,alphanumeric,casesensitive,reversed,textMatNames,numMatNames,sortlabels])	//sorts a 2D matrix
	variable dim,key			//dimension to sort and key row/col
	wave/Z numMat
	wave/Z/t textMat
	variable alphanumeric,casesensitive,reversed,sortlabels
	string textMatNames,numMatNames
	
	variable npts,i,text,nwaves
	
	newdatafolder/o root:Packages
	newdatafolder/o root:Packages:MatrixSort
	if(paramisdefault(textMat) & paramisdefault(numMat))
		abort "Specify matrix to sort!"
	else
		text=paramisdefault(numMat)
	endif
	if(text)
		npts=dimsize(textMat,dim)
		make/t/o/n=(npts) root:Packages:MatrixSort:sortWave
		wave/t textWave=root:Packages:MatrixSort:sortWave
		textWave[]=textMat[dim==0 ? p : key][dim==1 ? p : key]
	else
		npts=dimsize(numMat,dim)
		make/o/n=(npts) root:Packages:MatrixSort:sortWave
		wave numWave=root:Packages:MatrixSort:sortWave
		numWave[]=numMat[dim==0 ? p : key][dim==1 ? p : key]
	endif
	make/o/n=(npts) root:Packages:MatrixSort:indexWave
	wave indexWave=root:Packages:MatrixSort:indexWave
	indexWave=p
	if(alphanumeric & casesensitive & reversed & text)
		MakeIndex/A/C/R {textWave,indexWave},indexWave
	elseif(alphanumeric & reversed & text)
		MakeIndex/A/R {textWave,indexWave},indexWave
	elseif(casesensitive & reversed & text)
		MakeIndex/C/R {textWave,indexWave},indexWave
	elseif(alphanumeric & casesensitive & text)
		makeIndex/A/C {textWave,indexWave},indexWave
	elseif(alphanumeric & text)
		makeIndex/A {textWave,indexWave},indexWave
	elseif(casesensitive & text)
		makeIndex/C {textWave,indexWave},indexWave
	elseif(reversed & text)
		makeIndex/R {textWave,indexWave},indexWave
	elseif(text)
		makeIndex {textWave,indexWave},indexWave
	elseif(reversed)
		makeindex/R {numWave,indexWave},indexWave
	else
		makeindex {numWave,indexWave},indexWave
	endif
	MatrixIndexSort(dim,indexWave,textMat=textMat,numMat=numMat,sortLabels=sortLabels)
	if(!paramisdefault(textMatNames))
		nwaves=itemsinlist(textMatNames)
		for(i=0;i<nwaves;i+=1)
			MatrixIndexSort(dim,indexWave,textMat=$stringfromlist(i,textMatNames),sortLabels=sortLabels)
		endfor
	endif
	if(!paramisdefault(numMatNames))
		nwaves=itemsinlist(numMatNames)
		for(i=0;i<nwaves;i+=1)
			MatrixIndexSort(dim,indexWave,numMat=$stringfromlist(i,numMatNames),sortLabels=sortLabels)
		endfor
	endif
	
	killdatafolder root:Packages:MatrixSort
end

static Function MatrixIndexSort(dim,indexWave[,textMat,numMat,sortLabels])
	variable dim
	wave indexWave
	wave/Z numMat
	wave/Z/t textMat
	variable sortLabels
	
	variable npts,nops,i

	newdatafolder/o root:Packages
	newdatafolder/o root:Packages:MatrixIndexSort
	if(!paramisdefault(textMat))
		npts=dimsize(textMat,dim)
		nops=dimsize(textMat,dim==0)
		make/t/o/n=(npts) root:Packages:MatrixIndexSort:sortWave
		wave/t textWave=root:Packages:MatrixIndexSort:sortWave
		for(i=0;i<nops;i+=1)
			textWave[]=textMat[dim==0 ? p : i][dim==1 ? p : i]
			indexSort indexWave,textWave
			if(dim==0)
				textMat[][i]=textWave[p]
			elseif(dim==1)
				textMat[i][]=textWave[p]
			endif
		endfor
		if(sortLabels)
			textWave=getdimlabel(textMat,dim,p)
			indexSort indexWave,textWave
			for(i=0;i<npts;i+=1)
				setdimlabel dim,i,$textWave[i],textMat
			endfor
		endif
	elseif(!paramisdefault(numMat))
		npts=dimsize(numMat,dim)
		nops=dimsize(numMat,dim==0)
		make/o/n=(npts) root:Packages:MatrixIndexSort:sortWave
		wave numWave=root:Packages:MatrixIndexSort:sortWave
		for(i=0;i<nops;i+=1)
			numWave[]=numMat[dim==0 ? p : i][dim==1 ? p : i]
			indexSort indexWave,numWave
			if(dim==0)
				numMat[][i]=numWave[p]
			elseif(dim==1)
				numMat[i][]=numWave[p]
			endif
		endfor
		if(sortLabels)
			make/t/o/n=(npts) root:Packages:MatrixIndexSort:sortWave2
			wave/t textWave=root:Packages:MatrixIndexSort:sortWave2
			textWave=getdimlabel(numMat,dim,p)
			indexSort indexWave,textWave
			for(i=0;i<npts;i+=1)
				setdimlabel dim,i,$textWave[i],numMat
			endfor
		endif
	endif
	
	killdatafolder root:Packages:MatrixIndexSort
end
	
