#pragma rtGlobals=1		// Use modern global access method.
#include "SSA"

menu "Analysis"
	"Singular Spectrums Analysis...",SSAdisplay()
end

function SSAdisplay()

//Starts the ball rolling for SSA routines.  Generates the EOFs (empirical orthogonal functions) 
//for the data, then asks the user what to do with them.

	string SCREEN=StringByKey("SCREEN1",IgorInfo(0))
	variable right=Str2Num(StringFromList(3,SCREEN,","))/2
	variable bottom=(Str2Num(StringFromList(4,SCREEN,","))/2
	string folder=getdatafolder(1)
	setdatafolder root:
	if(DataFolderExists("SSAjunk"))
		string OldJunk;prompt OldJunk,"Name for old SSAjunk folder:"
		do
			doprompt "SSAjunk folder already exists",OldJunk
			if(DataFolderExists(OldJunk))
				Continue
			endif
		while(0)
		RenameDataFolder SSAjunk,$OldJunk
	endif
	NewDataFolder/o/s SSAjunk
	make/d/o data,Cdw,Ldw,Edw,EOFfreq_dw,EOFfreqR2_dw,hiPCT_dw,loPCT_dw,Ldw_prime
	make/d/o Cnw,Lnw,Enw,EOFfreq_nw,EOFfreqR2_nw,hiPCT_nw,loPCT_nw,Lrw,Crw
	make/o signal_ID,EOFsym_dw,EOFsym_nw,EigOrd_dw,EigOrd_nw,nw_dw_index
	make/d/o PC,RC,signal
	variable/g confidence_interval=0.95,dimensionality=NaN,DataLength=NaN,EndAlgorithm=1
		//in the following, * can be d, n, or r.  d=data, n=noise, r=noise realizations
		//data=wave containing de-meaned data
		//C*w=lag-covariance matrix, L*w=eigenvalues, E*w=EOFs (eigenvectors)
		//EOFfreq_*w, EOFfreqR2_*w=waves to hold dominant frequencies and R2 for fit of EOFs to sine waves
		//hiPCT_*w, loPCT_*w=waves to hold upper and lower limit of confidence interval
		//Ldw_prime=eigenspectrum for data projected on noise basis
		//signal_ID=index to EOFs - 1 means include EOF in noise model, 0 means treat EOF as part of signal
		//EOFsym_*w=index to symmetry of EOFs - 5 means symmetric about center, 7 means antisymmetric
		//				(numbers are chosen so that open squares and diamonds can be displayed)
		//EigOrd_*w=wave to use as ordinate in plot of eigenspectrum
		//nw_dw_index=index wave to convert EOF rank of data basis to closest 
		//				(or identical, if EOF is identified as signal) EOF rank of noise basis
		//PC=principle components;RC=reconstructed components;signal=reconstructed signal
	dowindow/k SSAcontrol		//kill old plot, then build new plot
	display/K=2/W=(10,40,right,bottom) Ldw vs EigOrd_dw as "SSA control"
	dowindow/c SSAcontrol
	appendtograph/c=(0,0,65535) Ldw_prime vs EigOrd_nw
	ModifyGraph margin=30,width=right-60,height=bottom-120
	ModifyGraph mode=3,mrkThick=0.5
	ModifyGraph zmrkNum(Ldw)={EOFsym_dw},zmrkSize(Ldw)={EOFfreqR2_dw,0,1,1,5}
	ModifyGraph zmrkNum(Ldw_prime)={EOFsym_nw},zmrkSize(Ldw_prime)={EOFfreqR2_nw,0,1,1,5}
	ModifyGraph minor=1,log(left)=1,mirror=1,tick=2,grid(bottom)=2,standoff=0
	ShowInfo;Ldw=NaN;Ldw_prime=NaN
	ControlBar 80
	TabControl Tabs pos={2.5,0},size={right-5,77.5},value=0,proc=SSA_tabs
	TabControl Tabs tabLabel(0)="Data"
	
	PopupMenu DataSelect pos={5,25},size={right/2-10,20}
	PopupMenu DataSelect mode=0,proc=SelectData,title="Select data wave:"
	PopupMenu DataSelect value=wavelist("*",";","DIMS:1")+"-;-;create test data;"
	SetVariable Mvalue pos={right/2+5,25},size={right/2-10,20},proc=ChangeM
	SetVariable Mvalue value=dimensionality,title="M (number of EOFs)="
	PopupMenu Ends pos={5,50},size={right/2-10,20},title="",mode=2,proc=SelectData
	PopupMenu Ends value="truncate matrix at data limits(BK);extend matrix beyond data (VG)"
	Button DoSSA pos={3*right/4+5,50},size={right/4-10,20}
	Button DoSSA title="Find EOFs",proc=SSA_decon
	
	PopupMenu NoiseType pos={5,25},size={right/2-10,20},title=""
	PopupMenu NoiseType mode=2,value="\\M1(White noise;Red noise"
	Button Noise pos={5,50},size={right/2-10,20}
	Button Noise title="Select noise/signal EOFs",proc=EOFselect
	Button DoNull pos={right/2+5,50},size={right/2-10,20}
	Button DoNull title="Create null hypothesis",proc=MakeNoise
	
	Button Rank pos={5,25},size={right/2-10,20}
	Button Rank title="Plot vs frequency",proc=PlotvsFreq
	Button Conf pos={right/2+5,25},size={right/2-10,20}
	Button Conf title="Plot confidence intervals",proc=ConfCalc
	SetVariable ConfIntHid pos={right/2+5,50},size={right/2-10,20}
	SetVariable ConfIntHid limits={0,0.999,0.05},title="Confidence interval:"
	SetVariable ConfIntHid value=confidence_interval,proc=ChangeConf,disable=1
	CheckBox DataBasis pos={5,50},size={right/4-10,20},title="Plot data basis"
	CheckBox DataBasis value=1,proc=PlotBasis
	CheckBox NoiseBasis pos={right/4+5,50},size={right/4-10,20},title="Plot noise basis"
	CheckBox NoiseBasis value=1,proc=PlotBasis
	
	Button Recon pos={right/2+5,50},size={right/2-10,20}
	Button Recon title="Reconstruct signal",proc=ReconSig
	
//	ValDisplay CsrAfreq pos={5,25},size={right/2-10,20},disable=1,value=1/xcsr(A)
//	ValDisplay CsrAfreq pos={5,25},size={right/2-10,20},disable=1,value=1/xcsr(B)
	
	SSA_tabs("Data",0)
	SetDataFolder $folder
end

Function SSA_tabs(name,tab)
	String name;Variable tab
	
	PopupMenu DataSelect,disable =(tab!=0)
	SetVariable Mvalue,disable=(tab!=0)
	PopupMenu Ends,disable=(tab!=0)
	Button DoSSA,disable=(tab!=0)
	PopupMenu NoiseType,disable=(tab!=1)
	Button Noise,disable=(tab!=1)
	Button DoNull,disable=(tab!=1)
	ControlInfo Freq
	if(V_flag==0)
		Button Rank,disable=(tab!=2)
	else
		Button Freq,disable=(tab!=2)
	endif
	ControlInfo NoConf
	if(V_flag==0)
		Button Conf,disable=(tab!=2)
	else
		Button NoConf,disable=(tab!=2)
	endif
	ControlInfo ConfInt
	if(V_flag!=0)
		SetVariable ConfInt,disable=(tab!=2)
	endif
	CheckBox DataBasis,disable=(tab!=2)
	CheckBox NoiseBasis,disable=(tab!=2)
	Button Recon,disable=(tab!=3)
	ValDisplay  CsrAfreq,disable=(tab!=4)
	ValDisplay  CsrBfreq,disable=(tab!=4)
end

Function SelectData(ctrlName,popNum,popStr):PopupMenuControl
	string ctrlName,popStr;variable popNum
	
	if(cmpstr(ctrlName,"DataSelect")==0)
		PopupMenu DataSelect mode=popNum
		if(cmpstr(popStr,"create test data")==0)
			AStest();popStr="data"
			PopupMenu DataSelect mode=(WhichListItem("data",wavelist("*",";","DIMS:1"))+1)
		endif
		wave/d dw=root:SSAjunk:data
		wave K=root:SSAjunk:signal_ID
		NVAR M=root:SSAjunk:dimensionality,N=root:SSAjunk:DataLength
		duplicate/d/o $popStr,dw;
		wavestats/q dw;dw-=V_avg
		N=dimsize(dw,0);M=floor(N/5)
		SetVariable Mvalue,limits={4,N-M,10}
		redimension/n=(M) K;K=1
	elseif(cmpstr(ctrlName,"Ends")==0)
		NVAR ends=root:SSAjunk:EndAlgorithm;ends=popNum-1
	endif
	TabControl Tabs tablabel(4)=""
	TabControl Tabs tablabel(3)=""
	TabControl Tabs tablabel(2)=""
	TabControl Tabs tablabel(1)=""
end

Function ChangeM(ctrlName,varNum,varStr,varName):SetVariableControl
	string ctrlName,varStr,varName;variable varNum
	
	wave K=root:SSAjunk:signal_ID,EigOrd=root:ssajunk:EigOrd_dw,NoiseOrd=root:SSAjunk:EigOrd_nw
	redimension/n=(varNum) K,EigOrd,NoiseOrd;K=1
end

Function SSA_decon(ctrlName):ButtonControl
	string ctrlName
	
	wave/d dw=root:SSAjunk:data,Cdw=root:SSAjunk:Cdw,Ldw=root:SSAjunk:Ldw,Edw=root:SSAjunk:Edw
	wave/d EOFfreq=root:SSAjunk:EOFfreq_dw,EOFfreqR2=root:SSAjunk:EOFfreqR2_dw
	wave EOFsym=root:SSAjunk:EOFsym_dw,EigOrd=root:SSAjunk:EigOrd_dw
	NVAR M=root:SSAjunk:dimensionality,N=root:SSAjunk:DataLength,ends=root:SSAjunk:EndAlgorithm
	SSA_EOF(N,M,ends,dw,Cdw,Ldw,Edw)	//Do SSA decomposition
	EOF_sym(Edw,EOFsym)					//Find EOF symmetries
	EigOrd=p+1							//Set EigOrd to rank number
	EOF_freq(Edw,EOFfreq,EOFfreqR2)		//Identify dominant frequencies asociated with EOFs
	EOFfreq/=deltax(dw)
	wavestats/q Ldw;SetAxis Bottom 0,(M)	//Set plot limits to match data
	SetAxis Left (10^(floor(log(V_min)))),(10^(ceil(log(V_max))))
	TabControl Tabs tablabel(1)="Null hypothesis"
	TabControl Tabs tablabel(2)="Plot",tablabel(3)="Reconstruct signal"
	TabControl Tabs tablabel(4)="Readout"
end

Function EOFselect(ctrlName):ButtonControl
	string ctrlName
	
	NVAR M=root:SSAjunk:dimensionality
	wave K=root:SSAjunk:signal_ID
	wave/d EOFfreq=root:SSAjunk:EOFfreq_dw,Edw=root:SSAjunk:Edw,Ldw=root:SSAjunk:Ldw
	string RECT=StringByKey("SCREEN1",IgorInfo(0))
	variable numcols=floor(Str2Num(StringFromList(3,RECT,","))/140)
	variable right=140*numcols+5
	variable bottom=(Str2Num(StringFromList(4,RECT,","))-40)
	variable left=5,top=max(bottom-ceil(15*(140*M/(right-5)))-60,0)
	dowindow/K EOFselection
	NewPanel/W=(left,top,right,bottom)/K=2 as "EOF signal/noise selection"
	dowindow/c EOFselection
	variable i;string EOFnum,titlestr
	TitleBox EOFselect pos={(right-left)/2-130,1},title="\Z14Select EOFs for noise reconstruction",frame=4
	Button Done pos={(right-left)/2-60,bottom-top-25},size={120,20},title="Done",proc=CloseControlPanel
	variable last=min(M,numcols*(bottom-top-60)/15)
	for(i=0;i<last;i+=1)
		EOFnum="EOF_"+num2str(i+1)
		sprintf titlestr,"%7s, freq=%5.4f",EOFnum,EOFfreq[i]
		CheckBox $EOFnum pos={140*mod(i,numcols),30+15*floor(i/numcols)}
		CheckBox $EOFnum value=K[i],proc=EOF_selection,title=titlestr
	endfor
end

Function EOF_selection(ctrlName,checked):CheckBoxControl
	string ctrlName;variable checked
	
	wave/d K=root:SSAjunk:signal_ID;variable EOFnum=NumberByKey("EOF",ctrlName,"_")-1
	K[EOFnum]=checked
end

Function CloseControlPanel(ctrlName):ButtonControl
	string ctrlName
	
	dowindow/c CloseControlPanel_tempwin;dowindow/k CloseControlPanel_tempwin
end

Function MakeNoise(ctrlName):ButtonControl
	string ctrlName
	
	NVAR M=root:SSAjunk:dimensionality,N=root:SSAjunk:DataLength
	wave/d dw=root:SSAjunk:data,Cdw=root:SSAjunk:Cdw,Ldw=root:SSAjunk:Ldw
	wave/d Edw=root:SSAjunk:Edw,Cnw=root:SSAjunk:Cnw,Lnw=root:SSAjunk:Lnw
	wave/d Enw=root:SSAjunk:Enw,Ldw_prime=root:SSAjunk:Ldw_prime
	wave K=root:SSAjunk:signal_ID,NDind=root:SSAjunk:nw_dw_index
	wave EigOrd=root:SSAjunk:EigOrd_nw,EOFsym=root:SSAjunk:EOFsym_nw
	wave/d EOFfreq=root:SSAjunk:EOFfreq_nw,EOFfreqR2=root:SSAjunk:EOFfreqR2_nw
	nvar Prob=root:SSAjunk:confidence_interval
	Noise_EOF(N,M,dw,K,Cdw,Ldw,Edw,Cnw,Lnw,Enw,Ldw_prime,NDind)
	redimension/n=(M) EigOrd;EigOrd[]=NDind[p]+1
	EOF_sym(Enw,EOFsym)
	EOF_freq(Enw,EOFfreq,EOFfreqR2)
	EOFfreq/=deltax(dw)
	wavestats/q Ldw_prime;variable Lmin=V_min,Lmax=V_max
	wavestats/q Ldw;Lmin=min(Lmin,V_min);Lmax=max(Lmax,V_max)
	SetAxis Left (10^(floor(log(Lmin)))),(10^(ceil(log(Lmax))))
	ControlInfo Rank
	if(V_flag==0)
		PlotvsFreq("Freq")
	else
		PlotvsFreq("Rank")
	endif
	ChangeConf("null",Prob,"null","null")
end

Function PlotvsFreq(ctrlName) : ButtonControl
	String ctrlName

//Toggles plot between Eigenvalue rank and associated frequency as ordinate.
//If frequency wave is not available, will call EOFfreq().

	wave EigOrd_dw=root:SSAjunk:EigOrd_dw,EigOrd_nw=root:SSAjunk:EigOrd_nw
	wave/d EOFfreq_dw=root:SSAjunk:EOFfreq_dw,EOFfreq_nw=root:SSAjunk:EOFfreq_nw
	wave NDind=root:SSAjunk:nw_dw_index
	nvar M=root:SSAjunk:dimensionality
	variable i
	if(cmpstr(ctrlName,"Rank")==0)
		Button Rank title="Plot vs rank",rename=Freq
		EigOrd_dw=EOFfreq_dw;EigOrd_nw=EOFfreq_nw
		SetAxis/a Bottom
	else
		Button Freq title="Plot vs frequency",rename=Rank
		EigOrd_dw[]=p+1;EigOrd_nw[]=NDind[p]+1
		SetAxis Bottom 0,(M)
	endif
End

Function ConfCalc(ctrlName):ButtonControl
	string ctrlName
	
	NVAR Prob=root:SSAjunk:confidence_interval
	wave/d hiPCT_dw=root:SSAjunk:hiPCT_dw,loPCT_dw=root:SSAjunk:loPCT_dw
	wave/d hiPCT_nw=root:SSAjunk:hiPCT_nw,loPCT_nw=root:SSAjunk:loPCT_nw
	if(cmpstr(ctrlName,"Conf")==0)
		Button Conf rename=NoConf,title="Remove confidence intervals"
		SetVariable ConfIntHid rename=ConfInt,disable=0
		ErrorBars/T=0.5/L=0.5 Ldw Y,wave=(hiPCT_dw,loPCT_dw)
		ErrorBars/T=0.5/L=0.5 Ldw_prime Y,wave=(hiPCT_nw,loPCT_nw)	
		ChangeConf("null",Prob,"null","null")
	else
		SetVariable ConfInt rename=ConfIntHid,disable=1
		Button NoConf rename=Conf,title="Plot confidence intervals"
		ErrorBars Ldw Off;ErrorBars Ldw_prime Off
	endif
end

Function ChangeConf(ctrlName,varNum,varStr,varName):SetVariableControl
	string ctrlName,varStr,varName;variable varNum
	
	wave/d dw=root:SSAjunk:data,Cnw=root:SSAjunk:Cnw,Enw=root:SSAjunk:Enw
	wave/d Cdw=root:SSAjunk:Cdw,Edw=root:SSAjunk:Edw,Ldw=root:SSAjunk:Ldw
	wave/d Ldw_prime=root:SSAjunk:Ldw_prime
	wave/d hiPCT_dw=root:SSAjunk:hiPCT_dw,loPCT_dw=root:SSAjunk:loPCT_dw
	wave/d hiPCT_nw=root:SSAjunk:hiPCT_nw,loPCT_nw=root:SSAjunk:loPCT_nw
	wave K=root:SSAjunk:signal_ID
	nvar ends=root:SSAjunk:endAlgorithm
	variable Prob=varNum,MC=1
	if(MC==2)
		SSA_MCsimple(dw,Cdw,Edw,ends,loPCT_dw,hiPCT_dw,Prob)
		SSA_MCsimple(dw,Cnw,Enw,ends,loPCT_nw,hiPCT_nw,Prob)
	else
		SSA_MCapprox(dw,Cdw,Edw,K,loPCT_dw,hiPCT_dw,Prob)
		SSA_MCapprox(dw,Cnw,Enw,K,loPCT_nw,hiPCT_nw,Prob)
	endif
	loPCT_dw=Ldw-loPCT_dw;hiPCT_dw=hiPCT_dw-Ldw
	loPCT_nw=Ldw_prime-loPCT_nw;hiPCT_nw=hiPCT_nw-Ldw_prime
end

Function PlotBasis(ctrlName,checked):CheckBoxControl
	string ctrlName;variable checked
	
	wave/d hiPCT_dw=root:SSAjunk:hiPCT_dw,loPCT_dw=root:SSAjunk:loPCT_dw
	wave/d hiPCT_nw=root:SSAjunk:hiPCT_nw,loPCT_nw=root:SSAjunk:loPCT_nw
	checked*=0.5;ControlInfo Conf
	if(cmpstr(ctrlName,"DataBasis")==0)
		ModifyGraph mrkThick(Ldw)=checked
		if(V_flag==0)
			ErrorBars/T=(checked)/L=(checked) Ldw Y,wave=(hiPCT_dw,loPCT_dw)
		endif
	else
		ModifyGraph mrkThick(Ldw_prime)=checked*0.5
		if(V_flag==0)
			ErrorBars/T=(checked)/L=(checked) Ldw_prime Y,wave=(hiPCT_nw,loPCT_nw)
		endif
	endif
end

Function ReconSig(ctrlName):ButtonControl
	string ctrlName
	
	wave/d Edw=root:SSAjunk:Edw,dw=root:SSAjunk:data
	wave/d RC=root:SSAjunk:RC,PC=root:SSAjunk:PC,sig=root:SSAjunk:signal
	wave K=root:SSAjunk:signal_ID
	nvar N=root:SSAjunk:DataLength,M=root:SSAjunk:dimensionality
	variable i
	SSA_PC_sig(N,M,Edw,dw,PC,K)
	SSA_RC_sig(N,M,Edw,PC,RC,K)
	redimension/n=(N) sig;sig=0
	for(i=0;i<M;i+=1)
		if(K[i]==0)
			sig[]+=RC[p][i]
		endif
	endfor
	copyscales dw,sig
	dowindow/F SSA_RCplot
	if(!V_flag)
		display dw as "Signal reconstruction";appendtograph/c=(0,0,0) sig
		dowindow/C SSA_RCplot
		Modifygraph minor=1,mirror=1,tick=2
	endif
end

function AStest()
	
	struct AR1_params AR1p
	string signal="signal",noise="noise",data="data"
	make/d/o/n=200 $signal=0,$noise=0,$data=0
	wave/d sig=$signal,noi=$noise,dat=$data
	string how;prompt how,"Use identical:",popup,"values;input parameters"
	doprompt "Recreate Allen & Smith's testwave",how
	if(cmpstr(how,"values")==0)
		sig[000,009]={+0.000,+0.000,+0.000,+0.000,+0.000,+0.000,-0.972,-0.185,+0.761,+0.785}
		sig[010,019]={-0.081,-0.799,-0.566,+0.292,+0.765,+0.341,-0.442,-0.674,-0.129,+0.527}
		sig[020,029]={+0.544,-0.056,-0.554,-0.393,+0.203,+0.530,+0.236,-0.306,-0.467,-0.089}
		sig[030,039]={+0.365,+0.377,-0.039,-0.384,-0.272,+0.140,+0.367,+0.164,-0.212,-0.324}
		sig[040,049]={-0.062,+0.253,+0.261,-0.027,-0.266,-0.189,+0.097,+0.255,+0.114,-0.147}
		sig[050,059]={-0.224,-0.043,+0.175,+0.181,-0.019,-0.184,-0.131,+0.067,+0.176,+0.079}
		sig[060,069]={-0.102,-0.155,-0.030,+0.122,+0.125,-0.013,-0.128,-0.091,+0.047,+0.122}
		sig[070,079]={+0.055,-0.071,-0.108,-0.021,+0.084,+0.087,-0.009,-0.089,-0.063,+0.032}
		sig[080,089]={+0.085,+0.038,-0.049,-0.075,-0.014,+0.058,+0.060,-0.006,-0.061,-0.044}
		sig[090,099]={+0.022,+0.059,+0.026,-0.034,-0.052,-0.010,+0.040,+0.042,-0.004,-0.043}
		sig[100,109]={-0.030,+0.016,+0.041,+0.018,-0.023,-0.036,-0.007,+0.028,+0.029,-0.003}
		sig[110,119]={-0.029,-0.021,+0.011,+0.028,+0.013,-0.016,-0.025,-0.005,+0.019,+0.020}
		sig[120,129]={-0.002,-0.020,-0.014,+0.007,+0.020,+0.009,-0.588,+0.470,+0.928,+0.306}
		sig[130,139]={-0.622,-0.786,-0.050,+0.695,+0.605,-0.164,-0.698,-0.408,+0.326,+0.643}
		sig[140,149]={+0.212,-0.431,-0.545,-0.035,+0.482,+0.420,-0.114,-0.484,-0.282,+0.226}
		sig[150,159]={+0.446,+0.147,-0.299,-0.378,-0.024,+0.334,+0.291,-0.079,-0.335,-0.196}
		sig[160,169]={+0.156,+0.309,+0.102,-0.207,-0.262,-0.017,+0.231,+0.202,-0.055,-0.232}
		sig[170,179]={-0.136,+0.108,+0.214,+0.071,-0.143,-0.181,-0.011,+0.160,+0.140,-0.038}
		sig[180,189]={-0.161,-0.094,+0.075,+0.148,+0.049,-0.099,-0.126,-0.008,+0.111,+0.097}
		sig[190,199]={-0.026,-0.112,-0.065,+0.052,+0.103,+0.034,-0.069,-0.087,-0.006,+0.077}
		noi[000,009]={-0.314,+0.141,-0.662,+0.173,+0.359,+1.356,+1.003,+0.071,-0.862,-0.777}
		noi[010,019]={-0.620,-1.088,-0.111,-0.077,+0.911,+1.058,+1.474,+0.230,-0.130,-1.121}
		noi[020,029]={-1.240,-2.026,-1.013,-0.979,-1.444,-1.214,-1.575,-2.512,-2.358,-2.440}
		noi[030,039]={-1.685,-2.267,-0.660,-1.603,-0.545,+0.011,-0.262,-0.223,-0.939,-0.120}
		noi[040,049]={+0.848,+2.263,+0.615,+1.314,+1.615,+0.634,+0.852,+1.260,-0.621,+0.861}
		noi[050,059]={-0.076,-0.687,-0.369,-0.698,+0.929,+0.123,-0.335,-1.352,-0.882,-1.399}
		noi[060,069]={-1.478,-1.003,-0.455,+1.568,+0.813,-0.277,+0.752,-0.885,-0.623,-1.142}
		noi[070,079]={-1.901,-2.417,-2.143,-1.425,-0.535,+0.351,-0.385,-0.776,-0.441,+0.368}
		noi[080,089]={-0.232,-0.458,-0.484,-1.016,+0.508,+1.335,+1.960,+1.811,+0.655,+0.698}
		noi[090,099]={+1.082,+0.150,-0.068,-0.253,-1.377,-1.226,-2.050,-1.572,-0.846,-1.329}
		noi[100,109]={-1.450,-0.602,-0.521,-0.131,-0.484,+0.066,+0.247,-0.717,-0.706,+0.035}
		noi[110,119]={+0.484,+1.654,+1.248,+0.786,+0.688,+0.884,+0.774,+1.311,+1.379,+1.346}
		noi[120,129]={+1.188,+0.723,+0.399,+0.484,+0.132,-0.228,-1.407,-0.249,-0.232,-0.210}
		noi[130,139]={+0.394,-0.010,+0.962,+0.726,-0.377,-0.190,-0.102,-0.514,-1.142,-0.451}
		noi[140,149]={-0.022,-0.109,+0.425,+0.020,+0.551,-0.378,-0.495,+0.023,-0.745,-0.291}
		noi[150,159]={-0.014,+1.463,+1.035,+0.781,-0.487,-0.441,-1.382,-1.311,+0.953,+0.425}
		noi[160,169]={+0.012,+0.466,-0.321,-0.832,+0.269,+0.262,+1.186,+1.900,+1.253,+1.749}
		noi[170,179]={+2.275,+1.440,+1.097,+0.008,+0.540,-0.378,-0.841,+0.055,-0.912,-0.069}
		noi[180,189]={-0.222,-0.722,-1.647,-0.974,-0.991,-0.104,-0.140,-0.297,+0.781,+2.122}
		noi[190,199]={+1.386,-0.269,+0.360,+0.298,+1.010,+0.992,+0.956,+1.457,+1.206,+1.645}
	else
		variable i,ph
		for(i=0;i<200;i+=1)
			if(abs(enoise(100))<=0.5)
				ph=enoise(pi)
				sig[i,199]+=sin((x-i)*pi/2.75+ph)/(e^((x-i)/30))
			endif
		endfor
		noi[0]=sqrt(1/(1-0.72^2))*gnoise(1)
		AR1p.gamm=0.72
		AR1p.alpha=1
		ar1p.u0=0
		noi[1,199]=AR1(noi[p-1],AR1p)
	endif
	dat=sig+noi
//	display sig;appendtograph/c=(0,0,65535) noi
//	appendtograph/c=(0,0,0) dat;modifygraph mirror=1,minor=1,tick=2
end
